@@ -56,10 +56,11 @@ impl FromStr for Lockfile {
     type Err = Error;
 
     fn from_str(toml_string: &str) -> Result<Self> {
-        Ok(toml::from_str(toml_string)?)
+        toml::from_str(toml_string).map_err(|e| Error::Parse(e.to_string()))
     }
 }
 
+#[allow(clippy::to_string_trait_impl)]
 impl ToString for Lockfile {
     fn to_string(&self) -> String {
         EncodableLockfile::from(self).to_string()
